package com.nisum.product_consumer.advice;

import com.nisum.product_consumer.exception.InvalidMessageException;
import com.nisum.product_consumer.exception.ProductProcessingException;
import com.nisum.product_consumer.service.KafkaConsumerServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import reactor.core.publisher.Mono;

import java.util.Map;

@RestControllerAdvice
//@Slf4j
public class GlobalExceptionHandler {

    private Logger log = LoggerFactory.getLogger(KafkaConsumerServiceImpl.class);

    @ExceptionHandler(InvalidMessageException.class)
    public Mono<Map<String, String>> handleInvalidMessageException(InvalidMessageException ex) {
        log.warn("Invalid Kafka message received: {}", ex.getMessage());
        return Mono.just(Map.of("error", ex.getMessage(), "status", "INVALID_MESSAGE"));
    }

    @ExceptionHandler(ProductProcessingException.class)
    public Mono<Map<String, String>> handleProductProcessingException(ProductProcessingException ex) {
        log.error("Product processing failed: {}", ex.getMessage());
        return Mono.just(Map.of("error", ex.getMessage(), "status", "PROCESSING_ERROR"));
    }

    @ExceptionHandler(Exception.class)
    public Mono<Map<String, String>> handleGenericException(Exception ex) {
        log.error("Unexpected error occurred: {}", ex.getMessage(), ex);
        return Mono.just(Map.of("error", "Internal Server Error", "status", "FAILURE"));
    }
}
