package com.nisum.product_consumer.service;

import com.nisum.product_consumer.dto.ProductDto;
import io.github.resilience4j.circuitbreaker.annotation.CircuitBreaker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;

@Service
public class ProductConsumerServiceImpl implements ProductConsumerService{

    private static final Logger logger = LoggerFactory.getLogger(ProductConsumerServiceImpl.class);
    private static final String CIRCUIT_BREAKER_INSTANCE = "productConsumer";

    @Autowired
    private WebClient webClient;

    @CircuitBreaker(name = CIRCUIT_BREAKER_INSTANCE, fallbackMethod = "fallbackGetAllProducts")
    @Override
    public Flux<ProductDto> getAllProductsFromProducer() {
        return webClient
                .get()
                .uri("/getAllProduct")
                .retrieve()
                .bodyToFlux(ProductDto.class)
                .doOnNext(product -> logger.info("Received product: {}", product));
    }

    public Flux<ProductDto> fallbackGetAllProducts(Throwable ex) {
        logger.error("Fallback triggered for getAllProductsFromProducer due to: {}", ex.getMessage());
        return Flux.empty();
    }
}
