package com.nisum.product_consumer.controller;

import com.nisum.product_consumer.dto.ProductDto;
import com.nisum.product_consumer.service.ProductConsumerService;
import com.nisum.product_consumer.service.ProductConsumerServiceImpl;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.*;
import org.mockito.junit.jupiter.MockitoExtension;
import org.reactivestreams.Publisher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

/*@WebFluxTest(ProductConsumerController.class)
@ExtendWith(MockitoExtension.class)*/

@ExtendWith(MockitoExtension.class)
public class ProductConsumerControllerTest {

    @InjectMocks
    private ProductConsumerServiceImpl productConsumerService;

    @Mock
    private WebClient.Builder webClientBuilder;

    @Mock
    private WebClient webClient;

    @Mock
    private WebClient.RequestHeadersUriSpec<?> requestHeadersUriSpec;

    @Mock
    private WebClient.RequestHeadersSpec<?> requestHeadersSpec;

    @Mock
    private WebClient.ResponseSpec responseSpec;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        when(webClientBuilder.baseUrl(Mockito.anyString())).thenReturn(webClientBuilder);
        when(webClientBuilder.build()).thenReturn(webClient);
    }

    @Test
    void testGetAllProductsFromProducer() {
        // Sample mock response
        ProductDto productDto1 = new ProductDto();
        productDto1.setId("101");
        productDto1.setName("iPhone 15");
        productDto1.setPrice(999.99);

        ProductDto productDto2 = new ProductDto();
        productDto2.setId("102");
        productDto2.setName("LED TV");
        productDto2.setPrice(1299.99);
        Flux<ProductDto> productDtoFlux = Flux.just(productDto1, productDto2);

        WebClient.RequestHeadersUriSpec requestHeadersUriSpec1 = requestHeadersUriSpec;
        when(webClient.get()).thenReturn(requestHeadersUriSpec1);
        when(requestHeadersUriSpec1.uri(Mockito.anyString())).thenReturn(requestHeadersSpec);
        when(requestHeadersSpec.retrieve()).thenReturn(responseSpec);
        when(responseSpec.bodyToFlux(ProductDto.class)).thenReturn(productDtoFlux);

        // Call the method
        Flux<ProductDto> result = productConsumerService.getAllProductsFromProducer();

        // Verify the result
        List<ProductDto> productList = result.collectList().block();
        StepVerifier.create(result)
                .expectNext(productList.get(0))
                .expectNext(productList.get(1))
                .verifyComplete();
    }

//    @Mock
//    private ProductConsumerService productConsumerService;
//
//    @InjectMocks
//    private ProductConsumerController productConsumerController;
//
//    @Mock
//    private WebTestClient webTestClient;
//
//    @BeforeEach
//    void setUp() {
//        MockitoAnnotations.openMocks(this);
//        webTestClient = WebTestClient.bindToController(productConsumerController).build();
//    }

//    @Test
//    public void fetchAllProductTest() {
//        ProductDto productDto1 = new ProductDto();
//        productDto1.setId("101");
//        productDto1.setName("iPhone 15");
//        productDto1.setPrice(999.99);
//
//        ProductDto productDto2 = new ProductDto();
//        productDto2.setId("102");
//        productDto2.setName("LED TV");
//        productDto2.setPrice(1299.99);
//
//        Flux<ProductDto> productDtoFlux = Flux.just(productDto1, productDto2);
//
//        when(productConsumerService.getAllProductsFromProducer()).thenReturn(productDtoFlux);
//        webTestClient.get()
//                .uri("/ProductConsumer/fetchAllProducts")
//                .accept(MediaType.APPLICATION_JSON)
//                .exchange()
//                .expectStatus().isOk()
//                .expectBodyList(ProductDto.class)
//                .value(products -> {
//                    assertThat(products).hasSize(2);
//                    assertThat(products.get(0).getId()).isEqualTo("101");
//                    assertThat(products.get(0).getName()).isEqualTo("iPhone 14");
//                    assertThat(products.get(1).getId()).isEqualTo("102");
//                    assertThat(products.get(1).getName()).isEqualTo("LED TV");
//                });
//
//
//        //Verify interactions
//        verify(productConsumerService, times(1)).getAllProductsFromProducer();
//    }
}


