package com.nisum.product_consumer.service;

import com.nisum.product_consumer.dto.ProductDto;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.*;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.test.StepVerifier;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.*;


@ExtendWith(MockitoExtension.class)
//@WebFluxTest(ProductConsumerService.class)
public class ProductConsumerServiceTest {

    @Mock
    private WebClient webClient; // Mock WebClient

    @Mock
    private WebClient.RequestHeadersUriSpec<?> requestHeadersUriSpec;

    @Mock
    private WebClient.RequestHeadersSpec<?> requestHeadersSpec;

    @Mock
    private WebClient.ResponseSpec responseSpec;

    @InjectMocks
    private ProductConsumerServiceImpl productConsumerService;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        // Manually creating mocks to resolve generic issues
        requestHeadersUriSpec = mock(WebClient.RequestHeadersUriSpec.class);
        requestHeadersSpec = mock(WebClient.RequestHeadersSpec.class);
        responseSpec = mock(WebClient.ResponseSpec.class);
    }

    // Given: Mock product list
    @Test
    public void getAllProductFromProducer() {
        ProductDto product1 = new ProductDto();//"101", "iPhone 14", 999.99
        product1.setId("101");
        product1.setName("iPhone 14");
        product1.setPrice(999.99);
        ProductDto product2 = new ProductDto();//"102", "Samsung S22", 899.99
        product2.setId("102");
        product2.setName("Samsung S22");
        product2.setPrice(899.99);

        Flux<ProductDto> mockProductFlux = Flux.just(product1, product2);

//        when(webClient.get()).thenReturn(requestHeadersUriSpec);  // Mock get()
//        when(requestHeadersUriSpec.uri(anyString())).thenReturn(requestHeadersSpec); // Mock uri()
//        when(requestHeadersSpec.retrieve()).thenReturn(responseSpec); // Mock retrieve()
//        when(responseSpec.bodyToFlux(ArgumentMatchers.any(Class.class))).thenReturn(mockProductFlux); // Mock bodyToFlux()

        WebClient.RequestHeadersUriSpec requestHeadersUriSpec1 = requestHeadersUriSpec;
        when(webClient.get()).thenReturn(requestHeadersUriSpec1);
        when(requestHeadersUriSpec1.uri(Mockito.anyString())).thenReturn(requestHeadersSpec);
        when(requestHeadersSpec.retrieve()).thenReturn(responseSpec);
        when(responseSpec.bodyToFlux(ProductDto.class)).thenReturn(mockProductFlux);

        Flux<ProductDto> result = productConsumerService.getAllProductsFromProducer();

        StepVerifier.create(result)
                .expectNext(product1)
                .expectNext(product2)
                .verifyComplete();

        // Verify interactions with WebClient
        verify(webClient, times(1)).get();
        verify(requestHeadersUriSpec, times(1)).uri("/getAllProduct");
        verify(requestHeadersSpec, times(1)).retrieve();
        verify(responseSpec, times(1)).bodyToFlux(ProductDto.class);



    }
}
